from __future__ import annotations

from .appserver_mixin import AppServerMixin


class HttpServerMixin(AppServerMixin):
    """Alias mixin for HTTP server transports using :class:`AppServerMixin`."""

    pass
